"use strict";

var contentTypeParser = require('content-type');

var humanReadableContentTypes = ['application/javascript', 'application/json', 'text/css', 'text/html', 'text/javascript', 'text/plain', 'text/xml'];

var getHeader = function getHeader(headers, headerName) {
  var value = headers[headerName];

  if (Array.isArray(value)) {
    return value[0];
  }

  return value;
};

var getContentType = function getContentType(res) {
  var contentType = getHeader(res.headers, 'content-type');

  if (!contentType) {
    return false;
  }

  return contentTypeParser.parse(contentType).type;
};
/**
 * Returns whether a request's body is human readable.
 *
 * @memberof module:node-vcr/media-type
 * @param { http.IncomingMessage } req
 * @returns { Boolean }
 */


var isHumanReadable = function isHumanReadable(res) {
  var contentType = getContentType(res);
  return humanReadableContentTypes.indexOf(contentType) >= 0;
};
/**
 * Returns whether a request's body is compressed.
 *
 * @memberof module:node-vcr/media-type
 * @param { http.IncomingMessage } res
 * @returns { Boolean }
 */


var isCompressed = function isCompressed(res) {
  var headers = res.headers;
  var encoding = getHeader(headers, 'content-encoding');
  return encoding && encoding !== 'identity';
};
/**
 * @module node-vcr/media-type
 */


module.exports = {
  getContentType: getContentType,
  isHumanReadable: isHumanReadable,
  isCompressed: isCompressed
};