"use strict";

var debug = require('debug')('node-vcr');

var http = require('http');

var https = require('https');

var followRedirects = require('follow-redirects');

var _require = require('util'),
    promisify = _require.promisify;

var url = require('url');

var modMapping = {
  http: http,
  https: https
  /**
   * Proxy `req` to `host` and yield the response.
   *
   * @module node-vcr/proxy
   * @param { http.IncomingMessage } req
   * @param { Array.<Buffer> } body
   * @param { String } host
   * @param { Number } maxRedirects
   * @returns { Promise.<http.IncomingMessage> }
   */

};

var proxy = function proxy(req, body, host, maxRedirects, callback) {
  followRedirects.maxRedirects = maxRedirects;
  var uri = new url.URL(host);
  var protocol = uri.protocol.replace(':', ''); // if maxRedirects is enabled in settings the client still can disable
  // redirect for custom request by setting the maxredirects header to 0

  var redirect = maxRedirects > 0;

  if (req.headers.maxredirects === '0') {
    redirect = false;
  }

  var mod = redirect ? followRedirects[protocol] || followRedirects.http : modMapping[protocol] || http;
  var pReq = mod.request({
    hostname: uri.hostname,
    port: uri.port,
    method: req.method,
    path: req.url,
    headers: req.headers,
    servername: uri.hostname,
    rejectUnauthorized: false
  }, function (pRes) {
    return callback(null, pRes);
  });
  pReq.setHeader('host', uri.host);
  debug("req=".concat(req.url, " host=").concat(uri.host));
  body.forEach(function (buf) {
    pReq.write(buf);
  });
  pReq.end();
};

module.exports = promisify(proxy);