"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var pd = require('pretty-data').pd;

var fse = require('fs-extra');

var zlib = require('zlib');

var JSONBigInt = require('json-bigint');

var buffer = require('./buffer');

var _require = require('./media-type'),
    getContentType = _require.getContentType,
    isHumanReadable = _require.isHumanReadable,
    isCompressed = _require.isCompressed;

var _require2 = require('./template'),
    render = _require2.render;
/**
 * Record the http interaction between `req` and `res` to disk.
 * The format is a vanilla node module that can be used as
 * an http.Server handler.
 *
 * @module node-vcr/record
 * @param { http.ClientRequest } req
 * @param { http.IncomingMessage } res
 * @param { String } filename
 * @param { Array } ignoredHeaders
 * @param { String } reqBody
 * @returns { Promise.<String> }
 */


var record = function record(req, res, filename, ignoredHeaders, reqBody) {
  var stream = res;

  var finalIgnoredHeaders = _toConsumableArray(ignoredHeaders);

  if (isCompressed(res) && isHumanReadable(res)) {
    stream = res.pipe(zlib.createUnzip());
    finalIgnoredHeaders.push('content-encoding');
    finalIgnoredHeaders.push('content-length');
  }

  return buffer(stream).then(function (body) {
    var encoding = isHumanReadable(res) ? 'utf-8' : 'base64';
    var contentType = getContentType(res);

    if (contentType && contentType.indexOf('/') >= 0) {
      var parts = contentType.split('/');
      contentType = parts[parts.length - 1];
    }

    body = Buffer.concat(body).toString(encoding);
    var data = body;

    try {
      if (contentType === 'json') {
        data = JSON.stringify(JSONBigInt.parse(data), null, 2);
      } else {
        if (pd[contentType]) {
          data = pd[contentType](body);

          if (reqBody) {
            reqBody = pd[contentType](reqBody);
          }
        }
      }
    } catch (error) {
      data = body;
    }

    return render(req, res, data, encoding, finalIgnoredHeaders, reqBody);
  }).then(function (data) {
    return fse.outputFile(filename, data);
  }).then(function () {
    return filename;
  });
};

module.exports = record;