"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var _ = require('lodash');

var sanitizeHeader = function sanitizeHeader(header) {
  if (Array.isArray(header)) {
    header = header.map(function (value) {
      return value.replace(/'/g, "\\'");
    });
  } else if (_typeof(header) === 'object') {
    header = _.mapValues(header, sanitizeHeader);
  } else if (typeof header === 'string') {
    header = header.replace(/'/g, "\\'");
  }

  return header;
};

var sanitizeBody = function sanitizeBody(body) {
  return body.replace(/(\\|\$\{)/g, '\\$&');
};

var render = function render(req, res, body, encoding, ignoredHeaders, reqBody) {
  var template = "const path = require('path')\n\n// ".concat(req.method, " ").concat(decodeURIComponent(req.path), "\n\n");
  Object.keys(req._headers).filter(function (key) {
    return ignoredHeaders.indexOf(key) === -1;
  }).forEach(function (key) {
    template += "// ".concat(key, ": ").concat(sanitizeHeader(req._headers[key]), "\n");
  });

  if (reqBody) {
    template += "\n// Request Body:\n";
    reqBody.split('\n').forEach(function (line) {
      template += "// ".concat(line, "\n");
    });
  }

  template += "\nmodule.exports = function (req, res) {\n  res.statusCode = ".concat(JSON.stringify(res.statusCode), "\n");
  Object.keys(res.headers).filter(function (key) {
    return ignoredHeaders.indexOf(key) === -1;
  }).forEach(function (key) {
    if (Array.isArray(res.headers[key])) {
      template += "\n  res.setHeader('".concat(key, "', ['").concat(sanitizeHeader(res.headers[key]).join("', '"), "'])");
    } else {
      template += "\n  res.setHeader('".concat(key, "', '").concat(sanitizeHeader(res.headers[key]), "')");
    }
  });
  template += "\n\n  res.setHeader('x-node-vcr-tape', path.basename(__filename, '.js'))\n";
  template += "\n  res.write(Buffer.from(`".concat(sanitizeBody(body), "`, '").concat(encoding, "'))");
  template += "\n  res.end()\n\n  return __filename\n}\n";
  return template;
};

module.exports = {
  render: render
};